SystemOrganization addCategory: #'Ma-Client-Server-Tester-SUnit extensions'!
SystemOrganization addCategory: #'Ma-Client-Server-Tester-Tests'!

MaClientServerRequest subclass: #MacsTestRequest
	instanceVariableNames: 'object1 object2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Tests'!

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 11:29'!
initialize
	super initialize.
	object1 := DateAndTime 
		date: 
			(Date 
				newDay: (1 to: 365) atRandom
				year: (1900 to: 2100) atRandom)
		time: Time now.
	object2 := UUID new -> (1 to: 100000) atRandom asWords! !

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 10:57'!
object1
	^object1! !

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 10:57'!
object1: aDateAndTime
	object1 := aDateAndTime! !

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 10:57'!
object2
	^object2! !

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 10:58'!
object2: anAssociation
	object2 := anAssociation! !

MaObject subclass: #MacsTestResponse
	instanceVariableNames: 'response1 response2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Tests'!

!MacsTestResponse class methodsFor: 'as yet unclassified' stamp: 'cmm 11/9/2008 11:10'!
fromRequest: aMacsTestRequest 
	^ self new
		initializeFrom: aMacsTestRequest ;
		yourself! !

!MacsTestResponse methodsFor: 'as yet unclassified' stamp: 'cmm 11/9/2008 17:35'!
initializeFrom: aMacsTestRequest 
	response1 := DateAndTime 
		date: (aMacsTestRequest object1 asDate addDays: 1)
		time: aMacsTestRequest object1 asTime.
	response2 := aMacsTestRequest object2 value -> aMacsTestRequest object2 key! !

!MacsTestResponse methodsFor: 'accessing' stamp: 'cmm 11/9/2008 11:14'!
response1
	^response1! !

!MacsTestResponse methodsFor: 'accessing' stamp: 'cmm 11/9/2008 11:14'!
response2
	^response2! !

MaTestCase subclass: #MaNetworkTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-SUnit extensions'!

!MaNetworkTestCase commentStamp: '<historical>' prior: 0!
My subclasses are intended to test network programs.  I provide methods for communicating with other images and tunnelling requests to them.!

MaNetworkTestCase subclass: #MaClientServerTestCase
	instanceVariableNames: 'server busy mergedBag testObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Tests'!

!MaClientServerTestCase class methodsFor: 'accessing' stamp: 'cmm 2/7/2009 10:56'!
defaultServerPort
	^ 60948! !

!MaClientServerTestCase class methodsFor: 'initialize-release' stamp: 'cmm 2/7/2009 10:04'!
initialize
	super initialize.
	Smalltalk addToStartUpList: self! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/10/2008 11:28'!
newBagFrom: start to: stop 
	^ ((start to: stop) collect: [ : n | n asWords ]) asBag! !

!MaClientServerTestCase methodsFor: 'shared' stamp: 'cmm 11/10/2008 15:47'!
noteBusyWhile: aBlock 
	
	[ busy := true.
	aBlock value ] ensure: [ busy := false ]! !

!MaClientServerTestCase methodsFor: 'test object server' stamp: 'cmm 11/9/2008 11:35'!
objectProcessorBlock
	^ [ : aMacsTestRequest | MacsTestResponse fromRequest: aMacsTestRequest ]! !

!MaClientServerTestCase methodsFor: 'armored code' stamp: 'cmm 11/10/2008 14:52'!
playerProtocol
	^ super playerProtocol , self testServerObjectProtocol! !

!MaClientServerTestCase methodsFor: 'test UDP object server' stamp: 'cmm 1/1/2013 13:47'!
sendObjectsToUdpServer: aCollection 
	| myLink |
	myLink := MaNetworkServerLink location:
		(MaServerLocation
			host: NetNameResolver localHostName
			port: self class defaultServerPort).
	myLink
		 networkProtocol: #udp ;
		 protocol: self testServerObjectProtocol.
	[ self noteBusyWhile:
		[ aCollection do:
			[ : each | myLink submit: each ] ] ] fork! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 1/1/2013 13:47'!
sendOutUdpByteArrayRequestsFrom: start to: stop 
	"Send the server the numbers, in English words, from 500 to 1000."
	| bag mySocket |
	bag := self
		newBagFrom: start
		to: stop.
	mySocket := MaClientSocket location:
		(MaServerLocation
			host: NetNameResolver localHostName
			port: self class defaultServerPort).
	mySocket networkProtocol: #udp.
	[ busy := true.
	bag do:
		[ : each | mySocket
			sendData: each
			startingAt: 2
			count: each size - 2
			waitForReplyIn: nil ].
	busy := false ] fork! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 1/1/2013 13:47'!
startByteArrayRequests
	| mySocket requestByteArray response eachResponse randomStream byteArrayRandomizer requestSize largestTest |
	largestTest := 9000.
	randomStream := Random new.
	response := ByteArray new: 4000.	"<- purposely too small"
	requestByteArray := ByteArray new: largestTest.
	byteArrayRandomizer := 
	[ : size | 
	1 
		to: size
		do: 
			[ : index | 
			requestByteArray 
				at: index
				put: (randomStream nextInt: 256) - 1 ] ].
	mySocket := MaClientSocket location: 
		(MaServerLocation 
			host: NetNameResolver localHostName
			port: self class defaultServerPort).
	
	[ busy := true.
	1 
		to: 200
		do: 
			[ : count | 
			requestSize := randomStream nextInt: largestTest.
			byteArrayRandomizer value: requestSize.
			eachResponse := mySocket 
				sendData: requestByteArray
				startingAt: 1
				count: requestSize
				waitForReplyIn: response.
			self should: 
				[ (requestByteArray 
					copyFrom: 1
					to: requestSize) reverse = (eachResponse 
						copyFrom: 1
						to: requestSize) ] ].
	busy := false ] fork! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 11/10/2008 16:05'!
startByteArrayServer
	server := MaServerSocket new.
	server 
		processOn: self class defaultServerPort
		using: [ : requestLifecycle | requestLifecycle requestByteArray reverse ]! !

!MaClientServerTestCase methodsFor: 'test object server' stamp: 'cmm 3/28/2013 20:24'!
startObjectRequests
	| myLink |
	myLink := MaNetworkServerLink location:
		(MaServerLocation
			host: NetNameResolver localHostName
			port: self class defaultServerPort).
	"			interface: #udp"
	[ busy := true.
	200 timesRepeat:
		[ | eachRequest myResponse remoteResponse |
		eachRequest := MacsTestRequest new.
		remoteResponse := myLink submit: eachRequest.
		myResponse := self objectProcessorBlock value: eachRequest.
		self
			 assert: remoteResponse ~~ myResponse ;
			 assert: remoteResponse response1 = myResponse response1 ;
			 assert: remoteResponse response2 = myResponse response2 ].
	busy := false ] fork! !

!MaClientServerTestCase methodsFor: 'test object server' stamp: 'cmm 11/10/2008 16:01'!
startObjectServer
	server := MaNetworkRequestServer protocol: self testServerObjectProtocol.
	server 
		processOn: self class defaultServerPort
		using: self objectProcessorBlock! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/11/2008 16:27'!
startUdpByteArrayServer
	server := MaServerSocket new
		networkProtocol: #udp ;
		yourself.
	"Server will load this Bag."
	mergedBag := Bag new.
	server 
		processOn: self class defaultServerPort
		using: [ : requestLifecycle | mergedBag add: requestLifecycle requestByteArray asString ]! !

!MaClientServerTestCase methodsFor: 'test UDP object server' stamp: 'cmm 11/11/2008 16:27'!
startUdpObjectServer
	server := MaNetworkRequestServer protocol: self testServerObjectProtocol.
	server networkProtocol: #udp.
	testObjects := Bag new.
	server 
		processOn: self class defaultServerPort
		using: [ : eachMacsTestRequest | testObjects add: eachMacsTestRequest ]! !

!MaClientServerTestCase methodsFor: 'shared' stamp: 'cmm 11/9/2008 11:15'!
stopServer
	server shutdown! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 11/9/2008 11:15'!
testByteArrayLayer
	self
		remotePerform: #startByteArrayServer
			in: 'server' ;
		remotePerform: #startByteArrayRequests
			in: 'client1' ;
		remotePerform: #startByteArrayRequests
			in: 'client2' ;
		remotePerform: #waitUntilDone
			in: 'client1' ;
		remotePerform: #waitUntilDone
			in: 'client2'.
	self 
		remotePerform: #stopServer
		in: 'server'! !

!MaClientServerTestCase methodsFor: 'test object server' stamp: 'cmm 11/9/2008 17:25'!
testObjectLayer
	self
		remotePerform: #startObjectServer
			in: 'server' ;
		remotePerform: #startObjectRequests
			in: 'client1' ;
		remotePerform: #startObjectRequests
			in: 'client2' ;
		remotePerform: #waitUntilDone
			in: 'client1' ;
		remotePerform: #waitUntilDone
			in: 'client2'.
	self 
		remotePerform: #stopServer
		in: 'server'! !

!MaClientServerTestCase methodsFor: 'shared' stamp: 'cmm 11/10/2008 14:51'!
testServerObjectProtocol
	^ { 
		MacsTestRequest.
		DateAndTime.
		Duration.
		Association.
		UUID.
		"String not needed, it is supported automatically."
		MacsTestResponse
	 }! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/10/2008 11:32'!
testUdpByteArrayLayer
	self
		remotePerform: #startUdpByteArrayServer
			in: 'server' ;
		remotePerform: #sendOutUdpByteArrayRequestsFrom:to:
			withArguments: #(500 1000 )
			in: 'client1' ;
		remotePerform: #sendOutUdpByteArrayRequestsFrom:to:
			withArguments: #(750 1250 )
			in: 'client2' ;
		remotePerform: #waitUntilDone
			in: 'client1' ;
		remotePerform: #waitUntilDone
			in: 'client2' ;
		remotePerform: #verifyMergedBag
			in: 'server' ;
		remotePerform: #stopServer
			in: 'server'! !

!MaClientServerTestCase methodsFor: 'test UDP object server' stamp: 'cmm 11/10/2008 15:58'!
testUdpObjectLayer
	| sampleObjects1 sampleObjects2 |
	sampleObjects1 := (1 to: 1000) collect: [ : e | MacsTestRequest new ].
	sampleObjects2 := (1 to: 1000) collect: [ : e | MacsTestRequest new ].
	self
		remotePerform: #startUdpObjectServer
			in: 'server' ;
		remotePerform: #sendObjectsToUdpServer:
			with: sampleObjects1
			in: 'client1' ;
		remotePerform: #sendObjectsToUdpServer:
			with: sampleObjects2
			in: 'client2' ;
		remotePerform: #waitUntilDone
			in: 'client1' ;
		remotePerform: #waitUntilDone
			in: 'client2' ;
		remotePerform: #verifyMostObjectsReceived:
			with: sampleObjects1 , sampleObjects2
			in: 'server' ;
		remotePerform: #stopServer
			in: 'server'! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/11/2008 16:52'!
verifyMergedBag
	| shouldBag |
	shouldBag := Bag new.
	{  (500 to: 1000). (750 to: 1250)  } do: 
		[ : eachRange | 
		eachRange first 
			to: eachRange last
			do: 
				[ : n | 
				| words |
				words := n asWords.
				shouldBag add: (words 
						copyFrom: 2
						to: words size - 1) ] ].
	"May we expect at least half of them to get through..?"
	self assert: mergedBag size > (shouldBag size * 0.5) truncated.
	self assert: (shouldBag maIncludesAll: mergedBag)! !

!MaClientServerTestCase methodsFor: 'test UDP object server' stamp: 'cmm 11/10/2008 15:58'!
verifyMostObjectsReceived: aCollection 
	self assert: testObjects size > (aCollection size * 0.8) truncated.
	testObjects do: 
		[ : eachReceivedUdp | 
		self assert: (aCollection anySatisfy: 
				[ : eachTestFixture | 
				eachTestFixture object1 = eachReceivedUdp object1 and: [ eachTestFixture object2 = eachReceivedUdp object2 ] ]) ]! !

!MaClientServerTestCase methodsFor: 'shared' stamp: 'cmm 8/24/2005 20:50'!
waitUntilDone

	[ busy ] whileTrue: [ (Delay forSeconds: 1) wait ].
	^ nil! !

!MaNetworkTestCase class methodsFor: 'overriding' stamp: 'cmm 1/2/2013 16:09'!
isAbstract
	^ super isAbstract or: [ self name = #MaNetworkTestCase ]! !

!MaNetworkTestCase class methodsFor: 'overriding' stamp: 'cmm 1/3/2013 15:08'!
suiteClass
	^ MaClientServerTestSuite! !

!MaNetworkTestCase methodsFor: 'accessing' stamp: 'cmm 1/3/2013 14:23'!
orchestra
	^ suite orchestra! !

!MaNetworkTestCase methodsFor: 'accessing' stamp: 'cmm 1/3/2013 14:27'!
orchestra: aMaNetworkOrchestra 
	suite orchestra: aMaNetworkOrchestra! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 1/3/2013 14:27'!
playerLinkNamed: aString 
	^ self orchestra playerNamed: aString! !

!MaNetworkTestCase methodsFor: 'accessing' stamp: 'cmm 1/5/2013 10:13'!
playerProtocol
	^ {MaRemotePerformRequest}! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 1/3/2013 14:27'!
playersDo: aBlock 
	 self orchestra players do: aBlock! !

!MaNetworkTestCase methodsFor: 'printing' stamp: 'cmm 5/3/2002 00:12'!
printOn: aStream

	testSelector
		ifNil: [ aStream nextPutAll: 'a', self class name ]
		ifNotNil: [ super printOn: aStream ]
! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 1/3/2013 14:27'!
pushClass: aClass 
	self orchestra players do:
		[ : each | each pushClass: aClass ]! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 4/23/2013 13:05'!
remotePerform: selectorSymbol in: playerName 
	^ self
		remotePerform: selectorSymbol
		withArguments: Array empty
		in: playerName! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 1/10/2013 13:10'!
remotePerform: selectorSymbol with: arg1 in: playerName 
	^ self
		remotePerform: selectorSymbol
		withArguments: {arg1}
		in: playerName! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 1/5/2013 16:23'!
remotePerform: selectorSymbol withArguments: anArray in: playerName 
	MaTestCaseNotification signal: 'remote performing ' , selectorSymbol , ' with arguments ' , anArray printString , ' in ' , playerName.
	^ (self playerLinkNamed: playerName)
		remotePerform: selectorSymbol
		withArguments: anArray! !

!MaNetworkTestCase methodsFor: 'server' stamp: 'cmm 1/3/2013 16:18'!
snapshot: shouldSave andQuit: shouldQuit 
	"Delay enough for response to be sent back to conductor."
	[ (Delay forSeconds: 1) wait.
	self orchestra homePlayer stop.
	Smalltalk
		snapshot: shouldSave
		andQuit: shouldQuit ] fork.
	^ nil! !

!MaNetworkTestCase methodsFor: 'server' stamp: 'cmm 1/3/2013 09:50'!
value: aMacsClientRequest 
	| result |
	result := aMacsClientRequest processFor: self.
	^ result == self ifFalse: [ result ]! !

TestSuite subclass: #MaNetworkTestSuite
	instanceVariableNames: 'orchestra'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-SUnit extensions'!

!MaNetworkTestSuite commentStamp: 'cmm 1/4/2013 16:47' prior: 0!
A MaNetworkTestSuite is a suite for testing network programs.  By employing a NetworkOrchestra made up of at least one server image and two client images, a fourth "test conductor" is able to direct a test suite that exercises the network program.!

MaNetworkTestSuite subclass: #MaClientServerTestSuite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Tests'!

!MaClientServerTestSuite methodsFor: 'setup-teardown' stamp: 'cmm 1/1/2013 15:00'!
playerNames
	^ #('client1' 'client2' 'server' )! !

!MaClientServerTestSuite methodsFor: 'overriding' stamp: 'cmm 3/22/2013 10:44'!
startingPort
	"Pick a port number range that won't conflict with the defaults, in case another application is running."
	^ 44420! !

!MaClientServerTestSuite methodsFor: 'setup-teardown' stamp: 'cmm 2/6/2009 17:19'!
suiteName
	^ 'macs'! !

!MaNetworkTestSuite methodsFor: 'setup-teardown' stamp: 'cmm 3/22/2013 10:44'!
createOrchestra
	| orch |
	orch := MaNetworkOrchestra new
		 name: '_' , self suiteName , 'Test' ;
		 nextPort: self startingPort ;
		 yourself.
	self playerNames do:
		[ : each | | testCase server player |
		testCase := self tests anyOne.
		player := MaNetworkPlayer new
			 name: each ;
			 server:
			(server := (MaNetworkRequestServer protocol: testCase playerProtocol)
				 processor: testCase ;
				 console: MaTranscriptConsole new ;
				 yourself) ;
			 yourself.
		orch addPlayer: player.
		"Don't let players timeout so we can debug, if necessary."
		player timeoutSeconds: 172800 ].
	^ orch! !

!MaNetworkTestSuite methodsFor: 'overriding' stamp: 'cmm 1/3/2013 14:38'!
debug
	self tests isEmpty ifTrue: [ ^ self ].
	[ self setUp ]
		on: MaNotification
		do:
			[ : noti | Transcript
				 cr ;
				 show: noti messageText.
			noti resume ].
	[ self orchestra isConductorImage ifTrue:
		[ | durationToRun |
		durationToRun := [ super debug ] durationToRun.
		Transcript
			 cr ;
			 show: 'Time to run all tests:' , durationToRun asString ] ] ensure:
		[ [ self tearDown ]
			on: MaNotification
			do:
				[ : noti | Transcript
					 cr ;
					 show: noti messageText.
				noti resume ].
		Transcript
			 cr ;
			 show: 'Done.' ]! !

!MaNetworkTestSuite methodsFor: 'setup-teardown' stamp: 'cmm 12/5/2011 21:46'!
findATranscript
	(Smalltalk
		at: #World
		ifAbsent: [  ]) ifNotNilDo: [ : world | world findATranscript: nil ]! !

!MaNetworkTestSuite methodsFor: 'setup-teardown' stamp: 'cmm 1/10/2013 12:46'!
launchOrchestra
	MaClientServerPreferences debug: true.
	(Smalltalk classNamed: #ScreenController) ifNotNilDo:
		[ : sc | ScreenController new fullScreenOff ].
	(orchestra := self createOrchestra)
		 deleteImages ;
		 launch.
	orchestra isConductorImage ifTrue: [ orchestra waitForAllPlayersReady ]! !

!MaNetworkTestSuite methodsFor: 'accessing' stamp: 'cmm 1/3/2013 15:29'!
orchestra
	^ orchestra! !

!MaNetworkTestSuite methodsFor: 'players' stamp: 'cmm 1/1/2013 15:00'!
playerNames
	self subclassResponsibility! !

!MaNetworkTestSuite methodsFor: 'setup-teardown' stamp: 'cmm 1/3/2013 17:24'!
setUp
	World color: ((Color random) adjustSaturation: -0.5 brightness: 0.0).
	MaClientServerPreferences debug: true.
	self launchOrchestra.
	self orchestra isConductorImage ifTrue: [ self findATranscript ]! !

!MaNetworkTestSuite methodsFor: 'accessing' stamp: 'cmm 2/6/2009 17:19'!
suiteName
	self subclassResponsibility ! !

!MaNetworkTestSuite methodsFor: 'setup-teardown' stamp: 'cmm 1/5/2013 09:58'!
tearDown
	orchestra shutDownPlayers! !

MaNotification subclass: #MaTestCaseNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-SUnit extensions'!

!TestResult methodsFor: '*ma-client-server-tester' stamp: 'cmm 4/18/2002 00:54'!
maDebugCase: aTestCase

	aTestCase runCase.
	self passed add: aTestCase
! !

!TestResult methodsFor: '*ma-client-server-tester' stamp: 'cmm 4/18/2002 00:56'!
maDebugCaseAsFailure: aTestCase

	aTestCase halt runCase.
	self passed add: aTestCase
! !
MaClientServerTestCase initialize!
